# coding: utf-8

import os
from datetime import datetime

import fabio
import numpy

from tomoscan.esrf.volume.edfvolume import EDFVolume
from tomoscan.esrf.volume.mock import create_volume

_data = create_volume(
    frame_dims=(20, 20), z_size=11
)  # z_size need to be at least 10 to check loading from file name works
for i in range(len(_data)):
    _data[i] += 1


def test_edf_writer_header(tmp_path):
    """
    Test writing an edf volume from several EDFVolume and reading the entire volume back
    """
    acquisition_dir = tmp_path / "acquisition"
    os.makedirs(acquisition_dir)
    volume_dir = str(acquisition_dir / "volume")
    os.makedirs(volume_dir)

    sub_data = _data[:5]
    header = {"time": datetime.now().timestamp()}
    volume = EDFVolume(
        folder=volume_dir,
        data=sub_data,
        header=header,
    )
    volume.save()
    first_frame_path = os.path.join(volume_dir, "volume_000000.edf")
    assert os.path.exists(first_frame_path)
    assert "time" in fabio.open(first_frame_path).header

    # test get_slice function
    numpy.testing.assert_array_equal(
        volume.get_slice(index=2, axis=0),
        sub_data[2],
    )
    numpy.testing.assert_array_equal(
        volume.get_slice(index=2, axis=1),
        sub_data[:, 2, :],
    )
    numpy.testing.assert_array_equal(
        volume.get_slice(index=2, axis=2),
        sub_data[:, :, 2],
    )
    volume.data = None

    numpy.testing.assert_array_equal(
        volume.get_slice(index=2, axis=0),
        sub_data[2],
    )
    numpy.testing.assert_array_equal(
        volume.get_slice(index=2, axis=1),
        sub_data[:, 2, :],
    )
    numpy.testing.assert_array_equal(
        volume.get_slice(index=2, axis=2),
        sub_data[:, :, 2],
    )

    assert volume.get_min_max_values() == (_data.min(), _data.max())
    assert volume.data_extension == "edf"
    assert volume.metadata_extension == "txt"



# coding: utf-8
from __future__ import annotations

import os
import tempfile

import pytest

from tomoscan.esrf.edfscan import EDFTomoScan
from tomoscan.esrf.scan.nxtomoscan import NXtomoScan
from tomoscan.esrf.mock import MockEDF
from tomoscan.factory import Factory
from tomoscan.scanbase import TomoScanBase
from tomoscan.tests.datasets import GitlabDataset


def test_scan_edf():
    """can we create a TomoScanBase object from a folder containing a
    valid .edf acquisition"""
    scan_dir = GitlabDataset.get_dataset("edf_datasets/test10")
    scan = Factory.create_scan_object(scan_dir)
    assert isinstance(scan, EDFTomoScan)


def test_one_nx():
    """Can we create a TomoScanBase from a .nx master file containing
    one acquisition"""
    master_file = GitlabDataset.get_dataset("h5_datasets/frm_edftomomill_oneentry.nx")
    scan = Factory.create_scan_object(master_file)
    assert isinstance(scan, NXtomoScan)
    assert scan.path == os.path.dirname(master_file)
    assert scan.master_file == master_file
    assert scan.entry == "/entry"


def test_one_two_nx():
    """Can we create a TomoScanBase from a .nx master file containing
    two acquisitions"""
    master_file = GitlabDataset.get_dataset("h5_datasets/frm_edftomomill_twoentries.nx")
    scan = Factory.create_scan_object(master_file)
    assert isinstance(scan, NXtomoScan)
    assert scan.path == os.path.dirname(master_file)
    assert scan.master_file == master_file
    assert scan.entry == "/entry0000"


def test_two_nx():
    """Can we create two TomoScanBase from a .nx master file containing
    two acquisitions using the Factory"""
    master_file = GitlabDataset.get_dataset("h5_datasets/frm_edftomomill_twoentries.nx")
    scans = Factory.create_scan_objects(master_file)
    assert len(scans) == 2
    for scan, scan_entry in zip(scans, ("/entry0000", "/entry0001")):
        assert isinstance(scan, NXtomoScan) is True
        assert scan.path == os.path.dirname(master_file)
        assert scan.master_file == master_file
        assert scan.entry == scan_entry


def test_invalid_path():
    """Insure an error is raised if the path as no meaning"""
    with pytest.raises(ValueError):
        Factory.create_scan_object("toto")

    with pytest.raises(ValueError):
        Factory.create_scan_objects("toto")

    with tempfile.TemporaryDirectory() as scan_dir:
        with pytest.raises(ValueError):
            Factory.create_scan_object(scan_dir)


def test_edf_scan_creation():
    with tempfile.TemporaryDirectory() as folder:
        scan_dir = os.path.join(folder, "my_scan")
        MockEDF.mockScan(scanID=scan_dir, nRecons=10)
        scan = Factory.create_scan_object(scan_path=scan_dir)
        assert isinstance(scan, EDFTomoScan)
        scans = Factory.create_scan_objects(scan_path=scan_dir)
        assert len(scans) == 1
        assert isinstance(scans[0], EDFTomoScan)
        dict_ = scan.to_dict()
        Factory.create_scan_object_frm_dict(dict_)
        # test invalid dict
        dict_[TomoScanBase.DICT_TYPE_KEY] = "tata"
        with pytest.raises(ValueError):
            Factory.create_scan_object_frm_dict(dict_)
        del dict_[TomoScanBase.DICT_TYPE_KEY]
        with pytest.raises(ValueError):
            Factory.create_scan_object_frm_dict(dict_)



# coding: utf-8

import os

import pytest

from tomoscan.esrf.edfscan import EDFTomoScan
from tomoscan.esrf.scan.nxtomoscan import NXtomoScan
from tomoscan.esrf.mock import MockEDF
from tomoscan.factory import Factory
from tomoscan.identifier import ScanIdentifier


def test_edf_identifier(tmp_path):
    """insure identifier is working for hdf5"""
    test_dir = tmp_path / "test_dir"
    os.makedirs(test_dir)

    scan = MockEDF.mockScan(
        scanID=str(test_dir),
        nRadio=12,
        dim=20,
    )
    identifier = scan.get_identifier()
    assert isinstance(identifier, ScanIdentifier)
    assert str(identifier).startswith("edf:")
    scan_from_identifier = EDFTomoScan.from_identifier(identifier)
    assert scan_from_identifier.path == scan.path
    scan_from_factory = Factory.create_tomo_object_from_identifier(
        identifier=str(identifier)
    )
    assert scan_from_factory.path == scan.path

    with pytest.raises(TypeError):
        NXtomoScan.from_identifier(identifier)

    # insure if TOMO_TYPE is not provided then it will still be considered as a scan
    identifier_as_str = str(identifier)
    identifier = ":".join(
        [identifier_as_str.split(":")[0], identifier_as_str.split(":")[-1]]
    )
    scan_from_factory = Factory.create_tomo_object_from_identifier(
        identifier=str(identifier)
    )
    assert scan_from_factory is not None



import logging
import traceback
import functools
import os
import io
from contextlib import contextmanager


depreclog = logging.getLogger("tomoscan.DEPRECATION")

deprecache = set([])


def deprecated_warning(
    type_,
    name,
    reason=None,
    replacement=None,
    since_version=None,
    only_once=True,
    skip_backtrace_count=0,
):
    """
    Function to log a deprecation warning

    :param type_: Nature of the object to be deprecated:
        "Module", "Function", "Class" ...
    :param name: Object name.
    :param reason: Reason for deprecating this function
        (e.g. "feature no longer provided",
    :param replacement: Name of replacement function (if the reason for
        deprecating was to rename the function)
    :param since_version: First *silx* version for which the function was
        deprecated (e.g. "0.5.0").
    :param only_once: If true, the deprecation warning will only be
        generated one time for each different call locations. Default is true.
    :param skip_backtrace_count: Amount of last backtrace to ignore when
        logging the backtrace
    """
    if not depreclog.isEnabledFor(logging.WARNING):
        # Avoid computation when it is not logged
        return

    msg = "%s %s is deprecated"
    if since_version is not None:
        msg += " since tomoscan version %s" % since_version
    msg += "."
    if reason is not None:
        msg += " Reason: %s." % reason
    if replacement is not None:
        msg += " Use '%s' instead." % replacement
    msg += "\n%s"
    limit = 2 + skip_backtrace_count
    backtrace = "".join(traceback.format_stack(limit=limit)[0])
    backtrace = backtrace.rstrip()
    if only_once:
        data = (msg, type_, name, backtrace)
        if data in deprecache:
            return
        else:
            deprecache.add(data)
    depreclog.warning(msg, type_, name, backtrace)


def deprecated(
    func=None,
    reason=None,
    replacement=None,
    since_version=None,
    only_once=True,
    skip_backtrace_count=1,
):
    """
    Decorator that deprecates the use of a function

    :param reason: Reason for deprecating this function
        (e.g. "feature no longer provided",
    :param replacement: Name of replacement function (if the reason for
        deprecating was to rename the function)
    :param since_version: First *silx* version for which the function was
        deprecated (e.g. "0.5.0").
    :param only_once: If true, the deprecation warning will only be
        generated one time. Default is true.
    :param skip_backtrace_count: Amount of last backtrace to ignore when
        logging the backtrace
    """

    def decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            deprecated_warning(
                type_="Function",
                name=func.__name__,
                reason=reason,
                replacement=replacement,
                since_version=since_version,
                only_once=only_once,
                skip_backtrace_count=skip_backtrace_count,
            )
            return func(*args, **kwargs)

        return wrapper

    if func is not None:
        return decorator(func)
    return decorator


def filter_esrf_mounting_points(path: str):
    """
    filter path like '/mnt/multipath-shares' or '/gpfs/easy'
    """
    parts = path.split(os.sep)
    if len(parts) > 4:
        if parts[0] == "" and parts[1] in ("gpfs", "mnt") and parts[3] == "data":
            new_parts = list(parts[0:1]) + list(parts[3:])
            path = os.sep.join(new_parts)
    return path


def _deprecate_url_in_signature(url, name="url"):
    if url is None:
        return
    deprecated_warning(
        type_="parameter", name=name, reason="about to be removed", since_version="2.3"
    )


@contextmanager
def catch_log_messages(logger_name=depreclog.name, level=logging.WARNING):
    # Create a buffer to capture log messages
    log_capture = io.StringIO()

    # Get the logger and add a handler to capture messages
    logger = logging.getLogger(logger_name)
    handler = logging.StreamHandler(log_capture)
    handler.setLevel(level)
    logger.addHandler(handler)

    try:
        yield log_capture
    finally:
        # Clean up: remove the handler
        logger.removeHandler(handler)



# coding: utf-8

from __future__ import annotations

import h5py
import numpy
import pytest

from nxtomo.application.nxtomo import ImageKey
from tomoscan.esrf.scan.mock import MockNXtomo as _MockNXtomo
from tomoscan.scanbase import ReducedFramesInfos
from tomoscan.esrf.scan.nxtomoscan import NXtomoScan
from nxtomo.utils.transformation import (
    DetXFlipTransformation,
    DetYFlipTransformation,
)
from nxtomo.nxobject.nxtransformations import NXtransformations


class MockNXtomo(_MockNXtomo):
    def __init__(
        self,
        scan_path,
        ini_dark: numpy.array | None,
        ini_flats: numpy.array | None,
        final_flats: numpy.array | None,
        dim: int,
        n_proj: int,
        count_time: numpy.array | None = None,
        machine_current: numpy.array | None = None,
    ):
        assert ini_dark is None or ini_dark.ndim == 3, "ini_dark should be a 3d array"
        assert ini_flats is None or ini_flats.ndim == 3, "ini_dark should be a 3d array"
        assert (
            final_flats is None or final_flats.ndim == 3
        ), "ini_dark should be a 3d array"
        self._ini_darks = ini_dark
        self._ini_flats = ini_flats
        self._final_flats = final_flats
        self._count_time = count_time
        super().__init__(
            scan_path=scan_path,
            dim=dim,
            create_ini_dark=ini_dark is not None,
            create_ini_flat=ini_flats is not None,
            create_final_flat=final_flats is not None,
            n_ini_proj=n_proj,
            n_proj=n_proj,
        )

        # append count_time and machine_current to the HDF5 file
        with h5py.File(self.scan_master_file, "a") as h5_file:
            entry_one = h5_file.require_group(self.scan_entry)
            if machine_current is not None:
                monitor_grp = entry_one.require_group("control")
                monitor_grp["data"] = machine_current
            # rewrite count_time
            if count_time is not None:
                instrument_grp = entry_one.require_group("instrument")
                detector_grp = instrument_grp.require_group("detector")
                if "count_time" in detector_grp:
                    del detector_grp["count_time"]
                detector_grp["count_time"] = count_time

    def add_initial_dark(self):
        for frame in self._ini_darks:
            self._append_frame(
                data_=frame.reshape(1, frame.shape[0], frame.shape[1]),
                rotation_angle=self.rotation_angle[-1],
                image_key=ImageKey.DARK_FIELD.value,
                image_key_control=ImageKey.DARK_FIELD.value,
                diode_data=None,
            )

    def add_initial_flat(self):
        for frame in self._ini_flats:
            self._append_frame(
                data_=frame.reshape(1, frame.shape[0], frame.shape[1]),
                rotation_angle=self.rotation_angle[-1],
                image_key=ImageKey.FLAT_FIELD.value,
                image_key_control=ImageKey.FLAT_FIELD.value,
                diode_data=None,
            )

    def add_final_flat(self):
        for frame in self._final_flats:
            self._append_frame(
                data_=frame.reshape(1, frame.shape[0], frame.shape[1]),
                rotation_angle=self.rotation_angle[-1],
                image_key=ImageKey.FLAT_FIELD.value,
                image_key_control=ImageKey.FLAT_FIELD.value,
                diode_data=None,
            )


@pytest.mark.parametrize(
    "lr_flip, ud_flip", ((False, False), (True, False), (False, True))
)
def test_reduce_hdf5(tmp_path, lr_flip, ud_flip):
    """insure calculation of dark and flats are valid for a default use case"""
    dim = 20
    folder_1 = tmp_path / "test1"
    folder_1.mkdir()
    n_proj = 12
    n_darks = 10
    darks = numpy.ones((n_darks, dim, dim), dtype="f")

    darks_count_time = numpy.linspace(
        1.0, 2.0, n_darks, endpoint=True, dtype=numpy.float32
    )
    darks_machine_current = numpy.linspace(
        12.2, 13.3, n_darks, endpoint=True, dtype=numpy.float32
    )

    flats_s1 = numpy.asarray(
        [
            numpy.zeros((dim, dim), dtype="f"),
            numpy.ones((dim, dim), dtype="f"),
            numpy.ones((dim, dim), dtype="f") + 1.0,
        ]
    )
    flats_s1_count_time = numpy.ones(3, dtype=numpy.float32)
    flats_s1_machine_current = numpy.array([14, 13.5, 13.2], dtype=numpy.float32)

    flats_s2 = numpy.asarray(
        [
            numpy.ones((dim, dim), dtype="f") + 10.0,
            numpy.ones((dim, dim), dtype="f") + 11.0,
            numpy.ones((dim, dim), dtype="f") + 12.0,
        ]
    )
    flats_s2_count_time = numpy.array([10.0, 2.0, 3.0], dtype=numpy.float32)
    flats_s2_machine_current = numpy.array([13.6, 13.8, 14.1], dtype=numpy.float32)

    projections_count_time = numpy.linspace(
        100, 200.0, endpoint=True, num=12, dtype=numpy.float32
    )
    projections_machine_current = numpy.linspace(
        13.2, 13.5, endpoint=True, num=12, dtype=numpy.float32
    )

    count_time = numpy.concatenate(
        [
            darks_count_time,
            flats_s1_count_time,
            projections_count_time,
            flats_s2_count_time,
        ]
    )

    machine_current = numpy.concatenate(
        [
            darks_machine_current,
            flats_s1_machine_current,
            projections_machine_current,
            flats_s2_machine_current,
        ]
    )

    scan = MockNXtomo(
        scan_path=folder_1,
        ini_dark=darks,
        ini_flats=flats_s1,
        final_flats=flats_s2,
        dim=dim,
        n_proj=n_proj,
        count_time=count_time,
        machine_current=machine_current,
    ).scan

    patch_detector_flip(scan=scan, lr_flip=lr_flip, ud_flip=ud_flip)

    assert scan.machine_current is not None
    assert scan.exposure_time is not None

    # test reduced frames

    numpy.testing.assert_array_equal(
        scan.compute_reduced_darks(reduced_method="median")[0],
        scan.compute_reduced_darks(reduced_method="mean")[0],
    )

    numpy.testing.assert_array_equal(
        scan.compute_reduced_flats(reduced_method="median")[n_darks],
        numpy.median(flats_s1, axis=0),
    )
    numpy.testing.assert_array_equal(
        scan.compute_reduced_flats(reduced_method="mean")[n_darks],
        numpy.mean(flats_s1, axis=0),
    )

    numpy.testing.assert_array_equal(
        scan.compute_reduced_flats(reduced_method="median")[n_darks + 3 + n_proj],
        numpy.median(flats_s2, axis=0),
    )
    numpy.testing.assert_array_equal(
        scan.compute_reduced_flats(reduced_method="mean")[n_darks + 3 + n_proj],
        numpy.mean(flats_s2, axis=0),
    )

    numpy.testing.assert_array_equal(
        scan.compute_reduced_flats(reduced_method="first", output_dtype=numpy.float64)[
            n_darks
        ],
        numpy.zeros((dim, dim), dtype=numpy.float64),
    )

    numpy.testing.assert_array_equal(
        scan.compute_reduced_flats(reduced_method="last", output_dtype=numpy.uint8)[
            n_darks
        ],
        numpy.ones((dim, dim), dtype=numpy.uint8) + 1,
    )

    numpy.testing.assert_array_equal(
        scan.compute_reduced_flats(reduced_method="first", output_dtype=numpy.int16)[
            n_darks + 3 + n_proj
        ],
        numpy.ones((dim, dim), dtype=numpy.int16) + 10.0,
    )

    # test reduced metadata on darks
    _, darks_infos = scan.compute_reduced_darks(
        reduced_method="median", return_info=True
    )
    assert isinstance(darks_infos, ReducedFramesInfos)
    assert len(darks_infos.machine_current) == len(darks_infos.count_time) == 1
    numpy.testing.assert_almost_equal(
        darks_infos.machine_current[0], numpy.median(darks_machine_current)
    )
    numpy.testing.assert_almost_equal(
        darks_infos.count_time[0], numpy.median(darks_count_time)
    )

    _, darks_infos = scan.compute_reduced_darks(reduced_method="mean", return_info=True)
    numpy.testing.assert_almost_equal(
        darks_infos.machine_current[0], numpy.mean(darks_machine_current)
    )
    numpy.testing.assert_almost_equal(
        darks_infos.count_time[0], numpy.mean(darks_count_time)
    )

    _, darks_infos = scan.compute_reduced_darks(
        reduced_method="first", return_info=True
    )
    numpy.testing.assert_almost_equal(
        darks_infos.machine_current[0], darks_machine_current[0]
    )
    numpy.testing.assert_almost_equal(darks_infos.count_time[0], darks_count_time[0])

    _, darks_infos = scan.compute_reduced_darks(reduced_method="last", return_info=True)
    numpy.testing.assert_almost_equal(
        darks_infos.machine_current[0], darks_machine_current[-1]
    )
    numpy.testing.assert_almost_equal(darks_infos.count_time[0], darks_count_time[-1])

    # test reduced metadata on flats
    _, flats_infos = scan.compute_reduced_flats(
        reduced_method="median", return_info=True
    )
    assert isinstance(flats_infos, ReducedFramesInfos)
    assert len(flats_infos.machine_current) == len(flats_infos.count_time) == 2
    numpy.testing.assert_almost_equal(
        flats_infos.machine_current[0],
        numpy.median(flats_s1_machine_current),
    )
    numpy.testing.assert_almost_equal(
        flats_infos.machine_current[1],
        numpy.median(flats_s2_machine_current),
    )
    numpy.testing.assert_almost_equal(
        flats_infos.count_time[0], numpy.median(flats_s1_count_time)
    )
    numpy.testing.assert_almost_equal(
        flats_infos.count_time[1], numpy.median(flats_s2_count_time)
    )

    _, flats_infos = scan.compute_reduced_flats(reduced_method="mean", return_info=True)
    numpy.testing.assert_almost_equal(
        flats_infos.machine_current[0], numpy.mean(flats_s1_machine_current)
    )
    numpy.testing.assert_almost_equal(
        flats_infos.machine_current[1], numpy.mean(flats_s2_machine_current)
    )
    numpy.testing.assert_almost_equal(
        flats_infos.count_time[0], numpy.mean(flats_s1_count_time)
    )
    numpy.testing.assert_almost_equal(
        flats_infos.count_time[1], numpy.mean(flats_s2_count_time)
    )
    # note: no test are done on the first and last but this is the same as for darks

    # test detector flips
    assert flats_infos.lr_flip == lr_flip
    assert flats_infos.ud_flip == ud_flip


def test_reduce_hdf5_fails(tmp_path):
    folder_1 = tmp_path / "test2"
    scan = MockNXtomo(
        scan_path=folder_1,
        ini_dark=None,
        ini_flats=None,
        final_flats=None,
        dim=20,
        n_proj=12,
    ).scan
    assert scan.compute_reduced_flats(reduced_method="first", return_info=False) == {}
    assert scan.compute_reduced_darks(reduced_method="last", return_info=False) == {}


def patch_detector_flip(scan: NXtomoScan, lr_flip: bool, ud_flip: bool):
    "function that patches an NXtomo' detector flips"

    mc_stas_transformation = NXtransformations()
    mc_stas_transformation.add_transformation(DetYFlipTransformation(flip=lr_flip))
    mc_stas_transformation.add_transformation(DetXFlipTransformation(flip=ud_flip))
    mc_stas_transformation.save(
        file_path=scan.master_file,
        data_path="/entry/instrument/detector/transformations",
        overwrite=True,
    )



# coding: utf-8

from tomoscan.esrf.identifier.folderidentifier import (
    BaseFolderAndfilePrefixIdentifierMixIn,
)
from tomoscan.identifier import ScanIdentifier, VolumeIdentifier
from tomoscan.utils import docstring


__all__ = ["EDFTomoScanIdentifier", "EDFVolumeIdentifier"]


class _BaseEDFIdentifier(BaseFolderAndfilePrefixIdentifierMixIn):
    """Identifier specific to EDF TomoScan"""

    @property
    @docstring(ScanIdentifier)
    def scheme(self) -> str:
        return "edf"


class EDFTomoScanIdentifier(_BaseEDFIdentifier, ScanIdentifier):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs, tomo_type=ScanIdentifier.TOMO_TYPE)

    @staticmethod
    def from_str(identifier):
        from tomoscan.esrf.scan.edfscan import EDFTomoScan

        return (
            BaseFolderAndfilePrefixIdentifierMixIn._from_str_to_single_frame_identifier(
                identifier=identifier,
                SingleFrameIdentifierClass=EDFTomoScanIdentifier,
                ObjClass=EDFTomoScan,
            )
        )


class EDFVolumeIdentifier(_BaseEDFIdentifier, VolumeIdentifier):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs, tomo_type=VolumeIdentifier.TOMO_TYPE)

    @staticmethod
    def from_str(identifier):
        from tomoscan.esrf.volume.edfvolume import EDFVolume

        return (
            BaseFolderAndfilePrefixIdentifierMixIn._from_str_to_single_frame_identifier(
                identifier=identifier,
                SingleFrameIdentifierClass=EDFVolumeIdentifier,
                ObjClass=EDFVolume,
            )
        )



from enum import Enum


class CoordinateSystem(Enum):
    """
    * ESRF coordinate system

                     Z axis
                            ^    Y axis
                            | /
          x-ray             |/
          -------->          ------> X axis

    * McStas coordinate system

                          Y axis
                            ^   X axis
                            |  /
          x-ray             | /
          -------->          ------> Z axis

    """

    McStas = "McStas"
    ESRF = "ESRF"



from tomoscan.utils.io import deprecated_warning

deprecated_warning(
    "Module",
    name="tomoscan.framereducerbase",
    reason="Have been replaced by a dedicated module",
    replacement="tomoscan.framereducer",
    only_once=True,
)

from tomoscan.framereducer import *  # noqa F401



import os
import time

import h5py
import numpy

from tomoscan.esrf.scan.nxtomoscan import NXtomoScan, ImageKey

proj_data = numpy.arange(1000, 1000 + 10 * 20 * 30).reshape(30, 10, 20)
proj_angle = numpy.linspace(0, 180, 30)
dark_value = 0.5
flat_value = 1
dark_data = numpy.ones((10, 20)) * dark_value
dark_angle = numpy.array(
    [
        0,
    ]
)
flat_data_1 = numpy.ones((10, 20)) * flat_value
flat_angle_1 = numpy.array(
    [
        0,
    ]
)

flat_data_2 = numpy.ones((10, 20)) * flat_value
flat_angle_2 = numpy.array(
    [
        90,
    ]
)
flat_data_3 = numpy.ones((10, 20)) * flat_value
flat_angle_3 = numpy.array(
    [
        180,
    ]
)

# data dataset
data = numpy.empty((34, 10, 20))
data[0] = dark_data
data[1] = flat_data_1
data[2:17] = proj_data[:15]
data[17] = flat_data_2
data[18:33] = proj_data[15:]
data[33] = flat_data_3


def create_arange_dataset(file_path):
    if os.path.exists(file_path):
        os.remove(file_path)

    with h5py.File(file_path, mode="a") as h5f:
        entry = h5f.require_group("entry0000")

        # rotation angle
        assert data.ndim == 3
        entry["instrument/detector/data"] = data
        rotation_angle = numpy.empty(34)
        rotation_angle[0] = dark_angle
        rotation_angle[1] = flat_angle_1
        rotation_angle[2:17] = proj_angle[:15]
        rotation_angle[17] = flat_angle_2
        rotation_angle[18:33] = proj_angle[15:]
        rotation_angle[33] = flat_angle_3

        entry["sample/rotation_angle"] = rotation_angle

        # image key / images keys
        image_keys = []
        image_keys.append(ImageKey.DARK_FIELD.value)
        image_keys.append(ImageKey.FLAT_FIELD.value)
        image_keys.extend([ImageKey.PROJECTION.value] * 15)
        image_keys.append(ImageKey.FLAT_FIELD.value)
        image_keys.extend([ImageKey.PROJECTION.value] * 15)
        image_keys.append(ImageKey.FLAT_FIELD.value)
        entry["instrument/detector/image_key"] = numpy.array(image_keys)
        entry["instrument/detector/image_key_control"] = numpy.array(image_keys)


file_path = "test.h5"
create_arange_dataset(file_path)

scan = NXtomoScan(file_path, "entry0000")
assert len(scan.projections) == 30
assert len(scan.flats) == 3
assert len(scan.darks) == 1

scan.set_reduced_darks(
    {
        0: dark_data,
    }
)

scan.set_reduced_flats(
    {
        1: flat_data_1,
        17: flat_data_2,
        33: flat_data_3,
    }
)


scan._flats_weights = scan._get_flats_weights()


t0 = time.time()
sinogram_old = scan._get_sinogram_ref_imp(line=5)
print(f"execution time of the old implementation: {time.time() - t0}")
t0 = time.time()
sinogram_new = scan.get_sinogram(line=5)
print(f"execution time of the new implementation: {time.time() - t0}")

# plot the sinogram

from silx.gui import qt
from silx.gui.plot import Plot2D

app = qt.QApplication([])

plot_old = Plot2D()
plot_old.addImage(sinogram_old)
plot_old.setWindowTitle("old sinogram")
plot_old.show()

plot_new = Plot2D()
plot_new.addImage(sinogram_new)
plot_new.setWindowTitle("new sinogram")
plot_new.show()

raw_sinogram = proj_data[:, 5, :]

plot_raw = Plot2D()
plot_raw.addImage(raw_sinogram)
plot_raw.setWindowTitle("raw sinogram")
plot_raw.show()

# TODO: get the one from tomwer to compare as well
try:
    from tomwer.core.scan.hdf5scan import NXtomoScan as TomwerNXtomoScan
except ImportError:
    pass
else:
    scan_t = TomwerNXtomoScan(file_path, "entry0000")

    scan_t.set_reduced_darks(
        {
            0: dark_data,
        }
    )

    scan_t.set_reduced_flats(
        {
            1: flat_data_1,
            17: flat_data_2,
            33: flat_data_3,
        }
    )
    sinogram_tomwer = scan_t.get_sinogram(line=5)

    plot_tomwer = Plot2D()
    plot_tomwer.addImage(sinogram_tomwer)
    plot_tomwer.setWindowTitle("tomwer sinogram")
    plot_tomwer.show()

# TODO: check all lines of the sinogram
corrected = (raw_sinogram - dark_value) / (flat_value - dark_value)
numpy.testing.assert_array_equal(corrected, sinogram_old)


app.exec_()
